from enum import Enum

from .load_shedding import (
    Area, Stage, StageError, ProviderError,
    get_areas, get_area_schedule, get_stage, get_stages, get_stage_forecast, get_area_forecast
)
from .providers import Province, coct, eskom, sepush


class Provider(Enum):
    ESKOM = 1
    COCT = 2
    SE_PUSH = 3

    def __call__(self, *args, **kwargs):
        return {
            Provider.ESKOM: eskom.Eskom(*args, **kwargs),
            Provider.COCT: coct.CoCT(*args, **kwargs),
            Provider.SE_PUSH: sepush.SePush(*args, **kwargs),
        }.get(self, None)

    def __str__(self):
        return {
            self.ESKOM: eskom.Eskom.name,
            self.COCT: coct.CoCT.name,
            self.SE_PUSH: sepush.SePush.name,
        }.get(self, "Unknown Provider")



#      A python library for getting Load Shedding schedules.
#      Copyright (C) 2021  Werner Pieterson
#
#      This program is free software: you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation, either version 3 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program.  If not, see <https://www.gnu.org/licenses/>.
from datetime import timezone, timedelta, datetime
from typing import List, Tuple

from load_shedding.libs import sepush, citypower
from load_shedding.providers import dict_list_to_obj_list, ProviderError, to_utc, Province
from load_shedding.providers import Area, Stage, Provider


class Area(Area):
    def __init__(self, /, **kwargs) -> object:
        self.id: str = kwargs.get("id", "")
        self.name: str = kwargs.get("name", "")
        self.municipality: str = kwargs.get("region", "")
        self.province: Province = Province.from_name(kwargs.get("ProvinceName", ""))


class SePush(Provider):
    name = "SePush"

    def __init__(self, token: str = "") -> object:
        if not token:
            raise ProviderError(f"Invalid Provider Token.")
        else:
            self.api = sepush.SePush(token)

    def get_areas(self, search_text: str = None, max_results: int = None) -> List[Area]:
        try:
            areas = self.api.areas_search(search_text)
        except Exception as e:
            raise ProviderError(e)
        else:
            areas = dict_list_to_obj_list(areas.get("areas"), Area)
            return filter_empty_areas(areas)

    def get_area_schedule(self, area: Area, stage: Stage) -> List[Tuple]:
        try:
            area_schedule = self.api.area(area_id=area.id)
        except Exception as e:
            raise ProviderError(e)
        else:
            schedule = []
            events = area_schedule.get("events", {})
            if not events:
                return schedule

            note = events[0].get("note")
            parts = str(note).split(" ")
            stage = Stage(int(parts[1]))

            sast = timezone(timedelta(hours=+2), 'SAST')
            for day in area_schedule.get("schedule", {}).get("days", []):
                date = datetime.strptime(day.get("date"), "%Y-%m-%d")
                stages = day.get("stages", [])
                if len(stages) < stage.value:
                    continue
                for slot in stages[stage.value]:
                    start_str, end_str = slot.strip().split("-")
                    start = datetime.strptime(start_str, "%H:%M").replace(
                        year=date.year, month=date.month, day=date.day, second=0, microsecond=0, tzinfo=sast
                    )
                    end = datetime.strptime(end_str, "%H:%M").replace(
                        year=date.year, month=date.month, day=date.day, second=0, microsecond=0, tzinfo=sast
                    )
                    if end < start:
                        end = end + timedelta(days=1)
                    schedule.append((start, end))

            schedule = to_utc(schedule)
            return schedule

    def get_stage(self, coct_stage: bool = False) -> Stage:
        try:
            data = self.get_stages()
            stage = data.get(Provider.ESKOM)
            if coct_stage:
                stage = data.get(Provider.COCT)
        except Exception as e:
            raise ProviderError(e)
        else:
            return stage

    def get_stages(self) -> dict:
        stages = {
            Provider.COCT: Stage.UNKNOWN,
            Provider.ESKOM: Stage.UNKNOWN,
        }
        try:
            data = self.api.status()
            status = data.get("status")

            eskom_stage = status.get("eskom", {}).get("stage", 0)
            stages[Provider.ESKOM] = Stage(int(eskom_stage))

            coct_stage = status.get("capetown", {}).get("stage", 0)
            stages[Provider.COCT] = Stage(int(coct_stage))
        except Exception as e:
            raise ProviderError(e)
        else:
            return stages

    def get_stage_forecast(self) -> List:
        """ Get Stage forecast from City Power."""
        try:
            stage_forecast = citypower.get_stage_forecast()

            for i in range(len(stage_forecast)):
                stage_forecast[i]["stage"] = Stage(stage_forecast[i].get("stage"))
                stage_forecast[i]["start_time"] = stage_forecast[i]["start_time"].astimezone(timezone.utc)
                stage_forecast[i]["end_time"] = stage_forecast[i]["end_time"].astimezone(timezone.utc)
        except Exception as e:
            raise ProviderError(e)
        else:
            return stage_forecast


# def stage_from_status(status: int) -> Stage:
#     return {
#         1: Stage.NO_LOAD_SHEDDING,
#         2: Stage.STAGE_1,
#         3: Stage.STAGE_2,
#         4: Stage.STAGE_3,
#         5: Stage.STAGE_4,
#         6: Stage.STAGE_5,
#         7: Stage.STAGE_6,
#         8: Stage.STAGE_7,
#         9: Stage.STAGE_8,
#     }.get(status, Stage.UNKNOWN)


def filter_empty_areas(suburbs: List[Area]) -> List[Area]:
    filtered: List[Area] = []
    for area in suburbs:
        if hasattr(area, "total") and not area.total:
            continue
        filtered.append(area)
    return filtered



#      A python library for getting Load Shedding schedules.
#      Copyright (C) 2021  Werner Pieterson
#
#      This program is free software: you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation, either version 3 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program.  If not, see <https://www.gnu.org/licenses/>.
from datetime import timezone
from typing import List, Tuple

from load_shedding.libs import coct
from load_shedding.providers import eskom, ProviderError, to_utc
from load_shedding.providers import Province, Stage, Area


class Area(Area):
    def __init__(self, /, **kwargs) -> object:
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.municipality = kwargs.get("municipality")
        self.province = kwargs.get("province")


class CoCT(eskom.Eskom):
    name = "City of Cape Town"

    def get_areas(self, search_text: str = None, max_results: int = None) -> List[Area]:
        try:
            areas: List[Area] = []
            for a in coct.Area:
                area = Area(
                    id=a.value,
                    name=str(a),
                    municipality="City of Cape Town",
                    province=Province.WESTERN_CAPE,
                )
                areas.append(area)
        except Exception as e:
            raise ProviderError(e)
        else:
            return areas

    def get_area_schedule(self, area: Area, stage: Stage) -> List[Tuple]:
        try:
            area_schedule = coct.get_schedule(coct.Area(area.id), stage)
        except Exception as e:
            raise ProviderError(e)
        else:
            area_schedule = to_utc(area_schedule)
            return area_schedule

    def get_stage(self, coct_stage: bool = False) -> Stage:
        """ Get Stage from CoCT API."""
        try:
            info = coct.get_info()
            stage = info[0].get("current_stage")
        except Exception as e:
            raise ProviderError(e)
        else:
            return Stage(stage)

    def get_stage_forecast(self) -> list:
        """ Get Stage forecast from CoCT."""
        try:
            info = coct.get_info()
            stage = info[0].get("current_stage")
            start_time = info[0].get("start_time")
            end_time = info[0].get("next_stage_start_time")

            next_stage = info[0].get("next_stage")
            next_start_time = info[0].get("next_stage_start_time")
            next_end_time = info[0].get("next_stage_start_time")
        except Exception as e:
            raise ProviderError(e) from e
        else:
            return [{
                "stage": Stage(stage),
                "start_time": start_time.astimezone(timezone.utc),
                "end_time": end_time.astimezone(timezone.utc),
            }, {
                "stage": Stage(next_stage),
                "start_time": next_start_time.astimezone(timezone.utc),
                "end_time": next_end_time.astimezone(timezone.utc),
            }]



#      A python library for getting Load Shedding schedules.
#      Copyright (C) 2021  Werner Pieterson
#
#      This program is free software: you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation, either version 3 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program.  If not, see <https://www.gnu.org/licenses/>.

import json
from datetime import datetime, timedelta, timezone
from enum import Enum
from typing import Any, Dict, List, Tuple

import certifi
import urllib3
from bs4 import BeautifulSoup


class EskomError(Exception):
    pass


class Province(Enum):
    EASTERN_CAPE = 1
    FREE_STATE = 2
    GAUTENG = 3
    KWAZULU_NATAL = 4
    LIMPOPO = 6
    MPUMALANGA = 5
    NORTH_WEST = 7
    NORTERN_CAPE = 8
    WESTERN_CAPE = 9

    def __str__(self):
        return {
            self.EASTERN_CAPE: "Eastern Cape",
            self.FREE_STATE: "Free State",
            self.GAUTENG: "Gauteng",
            self.KWAZULU_NATAL: "Kwa-Zulu Natal",
            self.LIMPOPO: "Limpopo",
            self.MPUMALANGA: "Mpumalanga",
            self.NORTH_WEST: "North West",
            self.NORTERN_CAPE: "Nortern Cape",
            self.WESTERN_CAPE: "Western Cape",
        }.get(self, "Unknown Province")


class LoadShedding:
    base_url = "https://loadshedding.eskom.co.za/LoadShedding"

    def find_suburbs(self, search_text: str, max_results: int = 10) -> List[Dict]:
        if not max_results:
            max_results = 10
        url = f"{self.base_url}/FindSuburbs?searchText={search_text}&maxResults={max_results}"
        data = _call(url)
        return json.loads(data)

    def get_municipalities(self, province: Province) -> List[Dict]:
        url = f"{self.base_url}/GetMunicipalities/?Id={province.value}"
        data = _call(url)
        return json.loads(data)

    def get_schedule(self, province: Province, suburb_id: int, stage: int) -> List[Tuple]:
        tot: int = 3252
        url = f"{self.base_url}/GetScheduleM/{suburb_id}/{stage}/{province}/{tot}"
        data = _call(url)
        return parse_schedule_data(data)

    def get_schedule_area_info(self, suburb_id: int) -> Dict:
        url = f"{self.base_url}/GetScheduleAreaInfo/?Id={suburb_id}"
        data = _call(url)
        return parse_area_info_data(data)

    def get_status(self) -> int:
        url = f"{self.base_url}/GetStatus"
        data = _call(url)
        return int(data)


def _call(url: str) -> Any:
    try:
        with urllib3.PoolManager(retries=urllib3.Retry(total=3), ca_certs=certifi.where()) as conn:
            r = conn.request('GET', url)
            if r.status != 200:
                raise urllib3.response.HTTPError(r.status)
            return r.data
    except Exception as e:
        raise EskomError(f"Eskom is unreachable. Check your connection.") from e


def parse_area_info_data(data: str) -> Dict:
    try:
        soup = BeautifulSoup(data, "html.parser")
        items = soup.find_all("div", attrs={"class": "areaInfoItem"})
        area_info = {
            "Province": {
                "Id": int(items[0].find("input", attrs={"id": "provinceId"}).get('value').strip()),
                "Name": items[0].find("input", attrs={"id": "province"}).get('value').strip(),
            },
            "Municipality": {
                "Id": int(items[1].find("input", attrs={"id": "municipalityId"}).get('value').strip()),
                "Name": items[1].find("input", attrs={"id": "municipality"}).get('value').strip(),
            },
            "Suburb": {
                "Id": int(items[2].find("input", attrs={"id": "suburbId"}).get('value').strip()),
                "Name": items[2].find("input", attrs={"id": "suburbName"}).get('value').strip(),
            },
            "Period": items[3].contents[2].strip().split("\xa0to\xa0"),
        }
    except Exception as e:
        raise EskomError(f"Unable to parse area info data.") from e
    else:
        return area_info


def parse_schedule_data(data: str) -> List[Tuple]:
    try:
        schedule = []
        soup = BeautifulSoup(data, "html.parser")
        days_soup = soup.find_all("div", attrs={"class": "scheduleDay"})

        sast = timezone(timedelta(hours=+2), 'SAST')
        now = datetime.now(sast)
        for day in days_soup:
            date_soup = day.find("div", attrs={"class": "dayMonth"})
            date_str = date_soup.get_text().strip()
            date = datetime.strptime(date_str, "%a, %d %b")
            date = date.replace(year=now.year)

            time_soup = day.find_all("a")
            for time_tag in time_soup:
                start_str, end_str = time_tag.get_text().strip().split(" - ")
                start = datetime.strptime(start_str, "%H:%M").replace(
                    year=date.year, month=date.month, day=date.day, second=0, microsecond=0, tzinfo=sast
                )
                end = datetime.strptime(end_str, "%H:%M").replace(
                    year=date.year, month=date.month, day=date.day, second=0, microsecond=0, tzinfo=sast
                )
                if end < start:
                    end = end + timedelta(days=1)
                schedule.append((start, end))
    except Exception as e:
        raise EskomError(f"Unable to parse schedule data.") from e
    else:
        return schedule



#      A python library for getting Load Shedding schedules.
#      Copyright (C) 2021  Werner Pieterson
#
#      This program is free software: you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation, either version 3 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program.  If not, see <https://www.gnu.org/licenses/>.

import errno
import json
import logging
import os
from datetime import datetime, timezone, timedelta
from json import JSONDecodeError
from typing import Any, Dict, Final, List, Tuple

from load_shedding.providers import (
    Area, Provider, ProviderError, Stage, StageError, datetime_to_isoformat, isoformat_to_datetime,
    citypower
)

logging.basicConfig(
    level=logging.DEBUG,
    format="%(asctime)s [%(levelname)s] %(message)s",
    handlers=[
        logging.StreamHandler()
    ]
)
logger = logging.getLogger(__name__)

CACHE_TTL: Final = 86400  # 24 hours
MAX_FORECAST_DAYS: int = 7


class LoadSheddingError(Exception):
    pass


def get_areas(provider: Provider, search_text: str = None, max_results: int = None) -> List[Area]:
    try:
        areas = provider.get_areas(search_text, max_results)
    except ProviderError as e:
        raise ProviderError(f"Unable to get areas from {provider.name}.") from e
    else:
        return areas


def get_stage(provider: Provider, coct_stage: bool = False) -> Stage:
    try:
        stage = provider.get_stage(coct_stage=coct_stage)
    except Exception as e:
        raise ProviderError(f"Unable to get stage from {provider.name}.") from e
    return stage


def get_stages(provider: Provider) -> List[Stage]:
    try:
        stages = provider.get_stages()
    except Exception as e:
        raise ProviderError(f"Unable to get stage from {provider.name}.") from e
    return stages


def get_stage_forecast(provider: Provider) -> List[Dict]:
    try:
        if not provider:
            provider = citypower.CityPower()
        stage_forecast = provider.get_stage_forecast()
    except Exception as e:
        raise ProviderError(f"Unable to get stage forecast from {provider.name}.") from e
    else:
        return stage_forecast


def get_area_forecast(area_schedule: List, planned: dict, now: datetime = datetime.now(timezone.utc)) -> List:
    area_forecast = []
    planned_start, planned_end = planned.get("start_time"), planned.get("end_time")

    for schedule in area_schedule:
        area_start, area_end = schedule[0], schedule[1]

        if area_start > now + timedelta(days=MAX_FORECAST_DAYS):
            continue

        if area_start < now:
            continue

        if area_start >= planned_end:
            continue
        if area_end <= planned_start:
            continue

        # Clip schedules that overlap forecast start_time and end_time
        if area_start <= planned_start and area_end <= planned_end:
            area_start = planned_start
        if area_start >= planned_start and area_end >= planned_end:
            area_end = planned_end

        if area_start == area_end:
            continue

        # if planned_start <= area_start <= planned_end and planned_start <= area_end <= planned_end:
        area_forecast.append({
            "stage": planned.get("stage"),
            "start_time": area_start.astimezone(now.tzinfo),
            "end_time": area_end.astimezone(now.tzinfo),
        })

    return area_forecast


def get_area_schedule(provider: Provider, area: Area, stage: Stage = None, cached: bool = True,
                      now=datetime.now(timezone.utc)) -> List[Tuple]:
    try:
        os.makedirs('.cache')
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise LoadSheddingError(f"Unable to make cache dir.") from e

    if stage in [None]:
        stage = get_stage(provider)

    if stage in [Stage.UNKNOWN, Stage.NO_LOAD_SHEDDING]:
        raise StageError(f"{stage}")

    cached_schedules = {}
    cached_schedule = []

    cache_file = f".cache/{area.province.value}_{area.id}.json"
    try:
        with open(cache_file, "r") as cache:
            for k, v in json.loads(cache.read()).items():
                cached_schedules[Stage(int(k)).value] = v

            cached_schedule = cached_schedules.get(stage.value, {})

        if cached and cached_schedule:
            updated_at = datetime.fromisoformat(cached_schedule.get("updated_at", None))

            cache_age = (now - updated_at)
            if cache_age.total_seconds() < CACHE_TTL:
                schedule = cached_schedule.get("schedule", {})
                schedule = isoformat_to_datetime(schedule)
                return schedule
    except (AttributeError, FileNotFoundError, IndexError, JSONDecodeError, ValueError) as e:
        logging.debug(f"Unable to get schedule from cache. %s", e, exc_info=True)

    try:
        schedule = provider.get_area_schedule(area=area, stage=stage)
    except Exception as e:
        if cached_schedule:
            return cached_schedule  # best effort
        raise ProviderError(f"Unable to get schedule from {provider.name}.") from e
    else:
        with open(cache_file, "w") as cache:
            cached_schedules[stage.value] = {
                "updated_at": str(now),
                "schedule": datetime_to_isoformat(schedule),
            }
            cache.write(json.dumps(cached_schedules))
        return schedule


def list_to_dict(schedule: list, now = datetime.now(timezone.utc)) -> Dict:
    schedule_dict = {}
    for item in schedule:
        start = item[0]
        end = item[1]

    schedule_dict[start.strftime("%Y-%m-%d")] = (
        start.replace(second=now.second).strftime("%H:%M"),
        end.replace(second=now.second).strftime("%H:%M"),
    )
    return schedule_dict


def dict_list_to_obj_list(data: List[Dict], T: Any) -> Any:
    objs: List = []
    for d in data:
        s = T(**d)
        objs.append(s)
    return objs



#      A python library for getting Load Shedding schedules.
#      Copyright (C) 2021  Werner Pieterson
#
#      This program is free software: you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation, either version 3 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program.  If not, see <https://www.gnu.org/licenses/>.

import json
from datetime import datetime, timedelta, timezone
from enum import Enum
from typing import Any, Dict, List, Tuple

import bs4
import certifi
import urllib3
from bs4 import BeautifulSoup, Tag


SAST = timezone(timedelta(hours=+2), 'SAST')


class CityPowerError(Exception):
    pass


base_url = "https://www.citypower.co.za"


def get_stage_forecast() -> Dict:
    url = f"{base_url}/customers/Pages/Load_Shedding_Downloads.aspx"
    data = _call(url)
    return parse_stage_forecast(data)


def _call(url: str) -> Any:
    try:
        with urllib3.PoolManager(retries=urllib3.Retry(total=3), ca_certs=certifi.where()) as conn:
            r = conn.request('GET', url)
            if r.status != 200:
                raise urllib3.response.HTTPError(r.status)
            return r.data
    except Exception as e:
        raise CityPowerError(f"CityPower is unreachable. Check your connection.") from e


def parse_stage_forecast(data: str) -> Dict:
    try:
        soup = BeautifulSoup(data, "html.parser")

        table_soup = soup.find("strong", text="Date").find_parent("table").find_next_sibling("table")
        forecast = []
        rows_soup = table_soup.find_all("tr")
        for row in rows_soup:
            cols_soup = row.find_all("td")
            next_stage = {}
            stage_date = None
            for c in range(len(cols_soup)):
                text = cols_soup[c].text
                text = text.strip().replace("\n", "").replace("\u200b", "")
                if not text:
                    break
                if c == 0:
                    text = text.replace("st", "").replace("nd", "").replace("rd", "").replace("th", "")
                    stage_date = datetime.strptime(text, "%d %B %Y")
                if c == 2:
                    text = text.replace("24:", "00:")
                    start_time = datetime.strptime(text, "%H:%M")
                    start_time = start_time.replace(year=stage_date.year, month=stage_date.month, day=stage_date.day, tzinfo=SAST)
                    next_stage["start_time"] = start_time
                if c == 3:
                    text = text.replace("24:", "00:")
                    end_time = datetime.strptime(text, "%H:%M")
                    end_time = end_time.replace(year=stage_date.year, month=stage_date.month, day=stage_date.day, tzinfo=SAST)
                    if end_time < start_time:
                        end_time = end_time + timedelta(days=1)
                    if end_time < datetime.now(tz=SAST):
                        next_stage = {}
                        break
                    next_stage["end_time"] = end_time
                if c == 4:
                    try:
                        next_stage["stage"] = int(text)
                    except ValueError as err:
                        next_stage["stage"] = 0
            if next_stage:
                forecast.append(next_stage)
    except Exception as e:
            raise CityPowerError(f"Unable to parse HTML.") from e
    else:
        return forecast




#      A python library for getting Load Shedding schedules.
#      Copyright (C) 2021  Werner Pieterson
#
#      This program is free software: you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation, either version 3 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program.  If not, see <https://www.gnu.org/licenses/>.
import http
import json
from typing import Any, Dict, List

import certifi
import urllib3


class SePushError(Exception):
    pass


ESKOM = "eskom"
CITY_OF_CAPE_TOWN = "capetown"


class SePush:
    base_url = "https://us-central1-sepush-app.cloudfunctions.net/sepush-serverless-business-api-staging-http/business/2.0"
    token = ""

    def __init__(self, token=None):
        if token:
            self.token = token

    def areas_search(self, text: str) -> List[Dict]:
        url = f"{self.base_url}/areas_search?text={text}"
        data = _call(url, self.token)
        return json.loads(data)

    def area(self, area_id: str) -> List[Dict]:
        url = f"{self.base_url}/area?id={area_id}"
        data = _call(url, self.token)
        return json.loads(data)

    def check_allowance(self) -> Dict:
        url = f"{self.base_url}/api_allowance"
        data = _call(url, self.token)
        return json.loads(data)

    def status(self) -> int:
        url = f"{self.base_url}/status"
        data = _call(url, self.token)
        return json.loads(data)


def _call(url: str, token: str) -> Any:
    try:
        with urllib3.PoolManager(retries=urllib3.Retry(total=3), ca_certs=certifi.where()) as conn:
            headers = {"Token": token}
            print("\nurl: ", url)
            r = conn.request('GET', url, headers=headers)
            if r.status != 200:
                raise urllib3.response.HTTPError(r.status)
            print("\ndata: ", r.data)
            return r.data
    except http.HTTPStatus.FORBIDDEN as e:
        raise SePushError(f"Invalid API key/token.") from e
    except http.HTTPStatus.TOO_MANY_REQUESTS as e:
        raise SePushError(f"API quota exceeded.") from e
    except Exception as e:
        raise SePushError(f"SePush is unreachable. Check your connection.") from e
